#include "PWMRegs.h"
#include "GPIORegs.h"
#include "types.h"
#include "Utils.h"

/*
 * PWM Timer is residing on the APB bus. 
 * For the time being - we assume that the clock used to drive this bus has 
 * 50MHZ frequency
*/
  
static U32 PCLK = 50000000;
static U32 g_Refs = 0; 

void BuzzerSet (U32 StepFreq, U32 ToothFreq, U32 DutyCycle)
{
	// Set TOUT0
	
	// Set prescaler
	
	// Set MUX divider
	
	// Start timer
	
	// Turn off manual update bit
	
	// Wait and listen to the buzzer
	Util_Getch ();
	
	// Now stop it.
}

void Timer_Bdelay_Micro (U32 Micros)
{
	U32 Prev;
	U32 Current;
	U32 ElapsedSteps = 0;
	U32 StepsInSwar = 0xFFFF;
	
	if (g_Refs++ == 0) /* Check reentrancy */
	{ 
		
		/*
		 * Set timer step period T for Timer 4 to 1 microsecond - Divide PCLK by 50
		*/
		*TCFG0 &= ~(0xff<<8);
		*TCFG0 |= 24<<8;			    //prescaler = 24+1
		*TCFG1 &= ~(0xf<<16);
		*TCFG1 |= TIMER4_MUX_1_2;		//mux = 1/2
		
		/*
		 * Tooth period cT is set by multiplying T by the number of countown steps.
		*/	
		*TCNTB4 = StepsInSwar;
		
		/*
		 * Activate timer 4.
		*/	
		
		*TCON |= (TIMER4_START | TIMER4_MAN_UPDATE | TIMER4_AUTO_RELOAD);	//autoload, update TCNTB4, start timer 4
		*TCON &= ~(TIMER4_MAN_UPDATE);		//clear manual update bit
	}
	
	Prev = *TCNTO4;
	
	while (ElapsedSteps < Micros)
	{
		Current = *TCNTO4;
		if (Current <= Prev)
		{
			ElapsedSteps += (Prev - Current);
		}
		else
		{
			ElapsedSteps += (StepsInSwar - (Current - Prev));
		}
		
		Prev = Current;
	}
	
	if (--g_Refs == 0)
	{
		/* Stop the timer since it is no more referenced */	
		*TCON &= ~(TIMER4_START | TIMER4_AUTO_RELOAD);
	}
}

void Timer_Bdelay_Milli (U32 Millis)
{
	U32 Prev;
	U32 Current;
	U32 ElapsedSteps = 0;
	U32 StepsInOneMilli = (PCLK>>3)/1000;
	U32 TotalSteps = StepsInOneMilli * Millis;
	

	/*
	 * Set timer step period T for Timer 4 - Divide PCLK by 8
	*/	
	*TCFG0 &= ~(0xff<<8);
	*TCFG0 |= 3<<8;			        //prescaler = 3+1
	*TCFG1 &= ~(0xf<<16);
	*TCFG1 |= TIMER4_MUX_1_2;		//mux = 1/2

	/*
	 * Tooth period cT is set by multiplying T by the number of countown steps.
	*/	
	*TCNTB4 = StepsInOneMilli;
	

	/*
	 * Activate timer 4.
	*/	
	
	*TCON |= (TIMER4_START | TIMER4_MAN_UPDATE | TIMER4_AUTO_RELOAD);	//autoload, update TCNTB4, start timer 4
	*TCON &= ~(TIMER4_MAN_UPDATE);		//clear manual update bit
	
	Prev = *TCNTO4;

	while (ElapsedSteps < TotalSteps)
	{
		Current = *TCNTO4;
		if (Current <= Prev)
		{
			ElapsedSteps += (Prev - Current);
		}
		else
		{
			ElapsedSteps += (StepsInOneMilli - (Current - Prev));
		}
		
		Prev = Current;
	}

	/* Stop the timer */	
	*TCON &= ~(TIMER4_START | TIMER4_AUTO_RELOAD);	
}



void TestBdelayMilli ()
{
	U32 Val;
	
	Util_Printf ("Enter the delay in milliseconds (decimal)...");	
	Val = Util_GetIntNum ();
	Timer_Bdelay_Milli (Val);	
}

void TestBdelayMicro ()
{
	U32 Val;
	
	Util_Printf ("Enter the delay in microseconds (decimal)...");	
	Val = Util_GetIntNum ();
	Timer_Bdelay_Micro (Val);	
}

void TestPWMBuzzer ()
{
	U32 StepFreq;
	U32 SawToothFrequency;
	U32 DutyCycle;	 
	
	Util_Printf ("Enter the input (step) frequency (decimal)...");	
	StepFreq = Util_GetIntNum ();
	Util_Printf ("Enter the tooth frequency (decimal)...");	
	SawToothFrequency = Util_GetIntNum ();
	Util_Printf ("Enter the duty cycle percentage (decimal)...");	
	DutyCycle = Util_GetIntNum ();
	
	BuzzerSet (StepFreq, SawToothFrequency, DutyCycle);	
}
